/*
 *  @title    User-level main file: TEMPLATE TO BE MODIFIED !
 *	@author   imperix SA (dev@imperix.ch)
 *	@file     user.c
 *	@comment  Core functions require that this file implements at least these functions :
 *			  USER_SAFE UserInit(void)
 *			  void UserError(int code);
 */

#include "Driver/peripherals.h"							// Available DSP peripherals
#include "Core/interrupts.h"							// Interrupt-related routines
#include "user.h"										// This file's header.
#include <math.h>
#include <stdio.h>
#include <time.h>

// Selector for the type of PWM modulation:
#define PD												// This can be changed to APOD, or POD

/**
 * Global variables
 * Please pay attention not to define variables that are unnecessarily global !
 */

int time_sin;
float Uref;												// 50Hz output sinus [1]
float Duty_A, Duty_B, Duty_C, Duty_D;					// 5 levels duty cycles [1]
float Iout, I1, I2, I3, I4, I5;							// Currents on the DC busses [A]
float I1c, I2c, I3c, I4c, I5c;							// Calculated current [A]
float Uout;												// Output voltage [V]

/**
 * The user should put here his/her initialization routines.
 * This function is called only once at startup.
 */
USER_SAFE UserInit(void)
{
	// Set a 20 kHz switching frequency on FREQGEN0:
	SetFreqGenPeriod(FREQGEN0, (int)(SWITCHING_PERIOD/FPGA_CLK_PERIOD));

	// Configure all PWM channels:
#ifdef PD												// Phase disposition PWM
	ConfigPWMChannel(0, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(1, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(2, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(3, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	
#elif APOD												// Alternate phase opposition disposition PWM
	ConfigPWMChannel(0, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(1, FREQGEN0, INVTRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(2, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(3, FREQGEN0, INVTRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	
#elif POD												// Phase opposition disposition PWM
	ConfigPWMChannel(0, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(1, FREQGEN0, TRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(2, FREQGEN0, INVTRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
	ConfigPWMChannel(3, FREQGEN0, INVTRIANGLE, (int)(DEADTIME/FPGA_CLK_PERIOD));
#endif
	
	// Immediately activate all PWM channels:
	ActivatePWMChannel(0);
	ActivatePWMChannel(1);
	ActivatePWMChannel(2);
	ActivatePWMChannel(3);

	// Configure all analog inputs:
	SetADCAdjustments(0, 0.0001316, 0);					// Output current
	SetADCAdjustments(1, 0.0005406, 0.25);				// 1st stage current
	SetADCAdjustments(2, 0.0005223, -0.5); 				// 2nd stage current
	SetADCAdjustments(3, 0.0005169, 0); 				// 3rd stage current
	SetADCAdjustments(4, 0.0005338, -0.55); 			// 4th stage current
	SetADCAdjustments(5, 0.0005353, 0); 				// 5th stage current

	// Configure UsrInt1 on FREQGEN0:
	RegisterExt1Interrupt(&UserInterrupt1, FREQGEN0, 0.0, 0);

	// Configure UsrInt2 on the CPU timer with a 10ms interrupt period:
	RegisterTimerInterrupt(&UserInterrupt2, 10000);
	
	// Configure the sampling in the middle of the period of FREQGEN0:
	ConfigSampling(FREQGEN0, 0.5);

	return SAFE;										// Return 0 if no error
}


/**
 * First user-level interrupt service routine.
 * Remember that it has the highest level of priority when mapped to Ext1, intermediate when
 * mapped to Ext2 and lowest when mapped to the CPU Timer.
 */
USER_SAFE UserInterrupt1(void)
{
	Uref = 0.7*sin(2*PI*OUTPUT_FREQUENCY*SWITCHING_PERIOD*time_sin); // Generate a sinus reference at 50Hz
	time_sin++;

	if(time_sin > ((SWITCHING_FREQUENCY/OUTPUT_FREQUENCY)-1)) {time_sin = 0;}

	// Modulation: compute the PWM duty-cycles
	// The following corresponds to a phase-disposition approach:
	Duty_A = (Uref-0.5)*2;
	Duty_B = (Uref)*2;
	Duty_C = (Uref+0.5)*2;
	Duty_D = (Uref+1.0)*2;

	// Apply duty cycles to the modulators:
	SetPWMDutyCycle(0,Duty_A);
	SetPWMDutyCycle(1,Duty_B);
	SetPWMDutyCycle(2,Duty_C);
	SetPWMDutyCycle(3,Duty_D);

	// Send the new modulation data to the modulators:
	UpdatePWMData();
	
	// Retrieve some measurements for processing/display purposes:
	Iout = GetADC(0);									// Output current
	I1 = GetADC(1);
	I2 = GetADC(2);
	I3 = GetADC(3);
	I4 = GetADC(4);
	I5 = GetADC(5);

	// Compute the reconstructed values of the intermediary currents:
	I1c = Iout * Duty_A;
	I2c = Iout * (Duty_B-Duty_A);
	I3c = Iout * (Duty_D-Duty_C);
	I4c = Iout * (1-Duty_D);

	// Display the following sampled values on the oscilloscope using the DAC outputs:
	SetDACVoltage(0, I1);								// Actual current measurement of the current on the top bus
	SetDACVoltage(1, I1c);								// Estimated/reconstructed for I1
	SetDACVoltage(2, I1-I1c);							// Estimated/reconstructed error
	SetDACVoltage(3, I2);								// Actual current measurement of the current on the second bus

    return SAFE;
}


/**
 * Second user-level interrupt service routine.
 * Remember that it has the highest level of priority when mapped to Ext1, intermediate when
 * mapped to Ext2 and lowest when mapped to the CPU Timer.
 */
USER_SAFE UserInterrupt2(void)
{
	return SAFE;
}


/*
 * This routine aims to handle critical errors by requesting an appropriate answer at the user level.
 * Typically, the application should be blocked and the core will wait for manual clearing of the error.
 */
void UserError(ERROR_SOURCE source)
{
}
